C
        SUBROUTINE EVALTF(PC,PR,PL,TFVAL,IBOUND,
     *           NUMTF,JS,IS,KS,J,I,K,NCOL,NROW,NLAY,
     *           NSCOL,NSROW,NSLAY,NNZER,NSCOORD,THCK,
     *           XFOR,XBAC,YFOR,YBAC)
C***********************************************************************
C  EVALTF EVALUATES ANY NONZERO TEST FUNCTIONS
C  AT (J+XHAT,I+YHAT,K+ZHAT)=(PC,PR,PL), WHERE 
C  -0.5 <= XHAT <= 0.5,
C  -0.5 <= YHAT <= 0.5,
C  -0.5 <= ZHAT <= 0.5.
C
C  TEST FUNCTION IS DETERMINED BY NSC, NSR AND NSL.  FOR INTERIOR CELLS
C  SURROUNDED BY ACTIVE NEIGHBORS ON UNIFORM GRID, TEST FUNCTION IS 
C  PRODUCT  
C  F(XHAT)*G(YHAT)*H(ZHAT) WHERE
C  F(XHAT)= 0                                      XHAT<= -0.5-1/NSC
C           NSC*XHAT + 0.5*(NSC+1)    -0.5-1/NSC<  XHAT<  -0.5+1/NSC
C           1                         -0.5+1/NSC<= XHAT<=  0.5-1/NSC
C          -NSC*XHAT + 0.5*(NSC+1)     0.5-1/NSC<  XHAT<   0.5+1/NSC
C           0                          0.5+1/NSC<= XHAT 
C  AND SIMILARLY FOR G AND H.
C
C  IF GRID IS NONUNIFORM, TEST FUNCTIONS ARE SCALED BY GRID RATIOS:
C  F(XHAT)=
C           0                          XHAT<= -0.5-(1/NSC)*DELCOL(JS-1)
C
C           1-F   FOR CELL AT JS-1   
C           -0.5-(1/NSC)*DELCOL(JS-1)< XHAT<  -0.5
C
C           2*NSC*(1-XBAC(JS))*XHAT + XBAC(JS) + NSC*(1-XBAC(JS))   
C                                -0.5<=XHAT< -0.5+(1/NSC)*DELCOL(JS)
C           1  0.5+(1/NSC)*DELCOL(JS)<=XHAT<= 0.5-(1/NSC)*DELCOL(JS)
C
C           2*NSC*(XFOR(JS)-1)*XHAT + XFOR(JS) + NSC*(1-XFOR(JS))
C              0.5-(1/NSC)*DELCOL(JS)< XHAT< 0.5
C
C           1-F   FOR CELL AT JS+1
C                                  0.5<=XHAT< 0.5+(1/NSC)*DELCOL(JS+1)
C           0  .5+(1/NSC)*DELCOL(JS+1)<=XHAT     
C
C  AND SIMILARLY FOR G AND H.
C
C  THE SUM OF THE VALUES OF THE TEST FUNCTIONS AT ANY POINT IN THE
C  TRANSPORT DOMAIN IS 1.
C
C  TEST FUNCTIONS EXTEND FROM CENTER OF BOUNDARY CELLS TO BOUNDARY AT
C  VALUE OF ONE.
C  ALL TEST FUNCTIONS ARE ZERO IN INACTIVE CELLS.
C
C  THERE IS NO TEST FUNCTION ASSOCIATED WITH AN INACTIVE CELL.  IN 
C  A CELL ADJACENT TO AN INACTIVE CELL, WHERE THE TEST FUNCTION, IF THE
C  CELL WERE ACTIVE, WOULD BE NOZERO, THAT VALUE IS DISTRIBUTED 
C  PROPORTIONALLY TO OTHER TEST FUNCTIONS WHICH ARE NONZERO AT THAT 
C  POINT.
C
C  EVALTF RETURNS:
C  NNZER  THE NUMBER OF NONZERO TEST FUNCTIONS
C  FOR EACH NONZERO TEST FUNCTION, N=1 TO NNZER
C  NUMTF(N)   NODE NUMBER ASSOCIATED WITH EACH FUNCTION
C  TFVAL(N)   VALUE FOR EACH TEST FUNCTION
C  NSCOORD(3,N)  TRANSPORT GRID COORDINATES OF EACH NODE, IF NOT TF1=1
C**********************************************************************
C
      DOUBLE PRECISION PRECNO
      parameter (precno=5.d-7)
      DIMENSION IBOUND(NCOL,NROW,NLAY),NUMTF(8),TFVAL(8),NSCOORD(3,8)
      DIMENSION NTEMP(8),TEMP(8),NACT(8),NSTEMP(3,8),
     *    THCK(NSCOL,NSROW,NSLAY),XFOR(NSCOL),XBAC(NSCOL),YFOR(NSROW),
     *    YBAC(NSROW)
      COMMON /SUBGRD/ ISCOL1,ISCOL2,ISROW1,ISROW2,ISLAY1,
     *  ISLAY2,ISUBGD
      COMMON /ELLAM/ CINV,RINV,BINV,HCINV,HRINV,HBINV,
     *               STINIT,ADINIT,STMASS,ADMASS,OLMASS,
     *               AZERO,
     *               NSC,NSR,NSL,NT,NCTF,NRTF,NLTF,
     *               NEIGHB(8,2),NSLOPE(3),
     *               IDTOP,IDMAX,NCOEF,NSCH,NSRH,NSLH
C
      XHAT=PC-J
      YHAT=PR-I
      ZHAT=PL-K
c
c      if(pr.gt.1d10) print *,pc,pr,pl,js,is,ks,j,i,k
c      if(abs(xhat).gt..5)  print *,'bad pc',pc,xhat
c      if(abs(yhat).gt..5)  print *,'bad pr',pr,yhat
c      if(abs(zhat).gt..5)  print *,'eval bad pl',pl,zhat
c      if ((js.gt.nscol).or.(is.gt.nsrow).or.(ks.gt.nslay))        
c     *   print *, 'evaltf error - point not insubgrid',
c     *             js,xhat,is,yhat,ks,zhat
c      if (iscol1+js-1.ne.j .or. isrow1+is-1.ne.i .or. islay1+ks-1.ne.k)
c     *   print *,'evaltf bad point',iscol1,isrow1,islay1,js,is,ks,j,i,k,
c     *           iscol1+js-1,isrow1+is-1,islay1+ks-1
C
      NODE=(KS-1)*NSROW*NSCOL+(IS-1)*NSCOL+JS
C
C
C  FIRST TEST FUNCTION ALWAYS FOR CURRENT NODE
      NSCOORD(1,1)=JS
      NSCOORD(2,1)=IS
      NSCOORD(3,1)=KS
      NUMTF(1)=NODE
C
C********************************************************************
C
C   POINT IN CELL CENTER
C   POINT IN CELL CENTER
      IF ((ABS(XHAT).LE..5-HCINV).AND.(ABS(YHAT).LE..5-HRINV).AND.
     *            (ABS(ZHAT).LE..5-HBINV)) GOTO 5000
c      IF ((ABS(XHAT).LE..5-CINV).AND.(ABS(YHAT).LE..5-RINV).AND.
c     *            (ABS(ZHAT).LE..5-BINV)) GOTO 5000 
C
C********************************************************************
C
C   HANDLE FACES,  EDGES AND CORNERS
C
C  ASSUMING NODE IS INTERIOR,
C  FIND NEIGHBOR WITH DIFFERENT X VALUE
C                     DIFFERENT X,Y VALUES
C                     DIFFERENT Y VALUE
C  EVALUATE PORTION OF THE PRODUCT TEST FUNCTION IN THE 
C                                                 X, Y OR Z DIRECTION
C  THEN ADJUST LAYER AND DO THE SAME
C
C***********************************************************************
      DO 15 N=2,8
      NTEMP(N)=NODE
C
15    CONTINUE
      IF (XHAT.LE.0) THEN
            NX=-1
            INCRE=-1
            XBNS=NCTF-NCTF*XBAC(JS)
            TCX=MIN(1.D0, XBNS*XHAT+XBAC(JS)+0.5D0*XBNS)
      ELSE
                  NX=1
            INCRE=1
            XFNS=NCTF*XFOR(JS)-NCTF
                  TCX=MIN(1.D0, XFNS*XHAT+XFOR(JS)-0.5D0*XFNS)
      ENDIF
C
      JB=J+NX
      JSB=JS+NX
      NTEMP(2)=NTEMP(2)+INCRE
      NTEMP(3)=NTEMP(3)+INCRE
      NTEMP(6)=NTEMP(6)+INCRE
      NTEMP(7)=NTEMP(7)+INCRE
C
      IF (YHAT.LE.0) THEN
            NY=-1
            INCRE=-NSCOL
            YBNS=NRTF-NRTF*YBAC(IS)
            TRY=MIN(1.D0, YBNS*YHAT+YBAC(IS)+0.5D0*YBNS)
      ELSE            
                  NY=1
            INCRE=NSCOL 
            YFNS=NRTF*YFOR(IS)-NRTF
            TRY=MIN(1.D0, YFNS*YHAT+YFOR(IS)-0.5D0*YFNS)
      ENDIF
C
      IB=I+NY
      ISB=IS+NY
      NTEMP(3)=NTEMP(3)+INCRE
      NTEMP(4)=NTEMP(4)+INCRE
      NTEMP(7)=NTEMP(7)+INCRE
      NTEMP(8)=NTEMP(8)+INCRE
C
      IF (ZHAT.LE.0) THEN
            NZ=-1
            INCRE=-NSCOL*NSROW
            ZBAC=TH(0,0,-1,JS,IS,KS,THCK,NSCOL,NSROW,NSLAY)
            ZBNS=NLTF-NLTF*ZBAC
            TLZ=MIN(1.D0, ZBNS*ZHAT+ZBAC+0.5D0*ZBNS)
      ELSE
            NZ=1
            INCRE=NSCOL*NSROW
            ZFOR=TH(0,0,1,JS,IS,KS,THCK,NSCOL,NSROW,NSLAY)
            ZFNS=NLTF*ZFOR-NLTF
            TLZ=MIN(1.D0, ZFNS*ZHAT+ZFOR-0.5D0*ZFNS)
      ENDIF
C
      KB=K+NZ
      KSB=KS+NZ
      NTEMP(5)=NTEMP(5)+INCRE
      NTEMP(6)=NTEMP(6)+INCRE
      NTEMP(7)=NTEMP(7)+INCRE
      NTEMP(8)=NTEMP(8)+INCRE
C
C   DETERMINE TRANSPORT GRID COORDINATES OF NEIGHBORING
C   NODES (IN CASE EVALTF IS CALLED BY BNDYTF)
C
      DO 85 N=2,3
      N1=N+2
      NSTEMP(1,N)=JSB
      NSTEMP(1,N1)=JS
85    CONTINUE
      NSTEMP(1,6)=JSB
      NSTEMP(1,7)=JSB
      NSTEMP(1,8)=JS
      NSTEMP(2,2)=IS
      DO 90 N=3,4
      N1=N+2
      NSTEMP(2,N)=ISB
      NSTEMP(2,N1)=IS
90    CONTINUE
      NSTEMP(2,7)=ISB
      NSTEMP(2,8)=ISB
      DO 95 N=1,4
      N1=N+4
      NSTEMP(3,N)=KS
      NSTEMP(3,N1)=KSB
95    CONTINUE
C
C   CALCULATE TEST FUNCTION VALUES
C
C   XHAT>=0
C
C      TLZ    = MIN(1.0,(.5*( NLTF ))*(-ABS( ZHAT )+.5)+.5) 
C      TRY    = MIN(1.0,(.5*( NRTF ))*(-ABS( YHAT )+.5)+.5) 
C      TCX    = MIN(1.0,(.5*( NCTF ))*(-ABS( XHAT )+.5)+.5) 

      TCXX   = 1.D0-TCX
      TRYY   = 1.D0-TRY 
      TLZZ   = 1.D0-TLZ 
C
      TFVAL(1)=TCX*          TRY*       TLZ
      TEMP(2)= TCXX*        TRY*       TLZ
      TEMP(3)= TCXX*         TRYY*           TLZ
      TEMP(4)= TCX*            TRYY*           TLZ
      TEMP(5)= TCX*           TRY*       TLZZ
      TEMP(6)= TCXX*        TRY*       TLZZ
      TEMP(7)= TCXX*         TRYY*           TLZZ
      TEMP(8)= TCX*            TRYY*           TLZZ
C
C   DETERMINE IF NEIGHBORS ARE ACTIVE, 
C   ASSUMING NODE IS INTERIOR TO FLOW GRID
C   THIS SHOULD BE A LEGITIMATE PROCEDURE, SINCE STORAGE IS NOT
C   ACCESSED OUTSIDE OF X ARRAY, BUT IF CODE HAS BEEN COMPILED WITH
C   SUBSCRIPT CHECKING, IT WILL BLOW UP HERE
C
c      NACT(2)=IBOUND(JB,I,K)
c      NACT(3)=IBOUND(JB,IB,K)
c      NACT(4)=IBOUND(J,IB,K)
c      NACT(5)=IBOUND(J,I,KB)
c      NACT(6)=IBOUND(JB,I,KB)
c      NACT(7)=IBOUND(JB,IB,KB)
c      NACT(8)=IBOUND(J,IB,KB)

      if (jb.gt.0 .and. jb.le.ncol) NACT(2)=IBOUND(JB,I,K)
      if (jb.gt.0 .and. jb.le.ncol .and. ib.gt.0
     *    .and. ib.le.nrow) NACT(3)=IBOUND(JB,IB,K)
      if (ib.gt.0 .and. ib.le.nrow) NACT(4)=IBOUND(J,IB,K)
      if (kb.gt.0 .and. kb.le.nlay) NACT(5)=IBOUND(J,I,KB)
      if (jb.gt.0 .and. jb.le.ncol .and. kb.gt.0 .and. kb.le.nlay)
     *        NACT(6)=IBOUND(JB,I,KB)
      if (jb.gt.0 .and. jb.le.ncol .and. ib.gt.0
     *    .and. ib.le.nrow .and. kb.gt.0 .and. kb.le.nlay)
     *         NACT(7)=IBOUND(JB,IB,KB)
      if (ib.gt.0 .and. ib.le.nrow.and. kb.gt.0 .and. kb.le.nlay)
     *       NACT(8)=IBOUND(J,IB,KB)
c  do checks for debugging
c      if (jb.gt.0 .and. jb.le.ncol) NACT(2)=IBOUND(JB,I,K)
c      if (jb.gt.0 .and. jb.le.ncol .and. ib.gt.0
c     *    .and. ib.le.nrow) NACT(3)=IBOUND(JB,IB,K)
c      if (ib.gt.0 .and. ib.le.nrow) NACT(4)=IBOUND(J,IB,K)
c      if (kb.gt.0 .and. kb.le.nlay) NACT(5)=IBOUND(J,I,KB)
c      if (jb.gt.0 .and. jb.le.ncol .and. kb.gt.0 .and. kb.le.nlay)
c     *        NACT(6)=IBOUND(JB,I,KB)
c      if (jb.gt.0 .and. jb.le.ncol .and. ib.gt.0
c     *    .and. ib.le.nrow .and. kb.gt.0 .and. kb.le.nlay)
c     *         NACT(7)=IBOUND(JB,IB,KB)
c      if (ib.gt.0 .and. ib.le.nrow.and. kb.gt.0 .and. kb.le.nlay)
c     *       NACT(8)=IBOUND(J,IB,KB)
C
C
C   DETERMINE IF ON SUBGRID BOUNDARY
C   CORNER, EDGE
C 
      IF ((JB.LT.ISCOL1).OR.(JB.GT.ISCOL2).OR.
     *           (ABS(XHAT).LE..5-CINV)) THEN
         NACT(2)=0
         NACT(3)=0
         NACT(6)=0
         NACT(7)=0
      ENDIF
C 
      IF ((IB.LT.ISROW1).OR.(IB.GT.ISROW2).OR.
     *          (ABS(YHAT).LE..5-RINV)) THEN
         NACT(3)=0
         NACT(4)=0
         NACT(7)=0
         NACT(8)=0
      ENDIF
C
      IF ((KB.LT.ISLAY1).OR.(KB.GT.ISLAY2).OR.
     *           (ABS(ZHAT).LE..5-BINV)) THEN
         NACT(5)=0
         NACT(6)=0
         NACT(7)=0
         NACT(8)=0
      ENDIF
C
C****************************************************************
C   DETERMINE ACTIVE NEIGHBORING CELLS
C
61000 NNZER=1
      NNAC=8

        sum=tfval(1)
        if ((sum.gt.1+precno).or.(sum.lt.0.0-precno)) then
         print *,'evaltf bad tf'
           stop
        endif

      DO 65 N=2,8

          sum=sum+temp(n)
          if ((temp(n).gt.1+precno).or.(temp(n).lt.0.0-precno)) then
          print *,'evaltf bad tf'
        stop
        endif

        IF (NACT(N).NE.0) THEN
            NNZER=NNZER+1
            NUMTF(NNZER)=NTEMP(N)
            DO 60 M=1,3
            NSCOORD(M,NNZER)=NSTEMP(M,N)
60          CONTINUE
            TFVAL(NNZER)=TEMP(N)
        ELSE
            NUMTF(NNAC)=NTEMP(N)
            TFVAL(NNAC)=TEMP(N)
            NNAC=NNAC-1
        ENDIF
65    CONTINUE
      if ((sum.gt.1+precno).or.(sum.lt.0.0-precno)) then
         print *,'evaltf tfs ne 1'
        stop
        endif
C
C   DISTRIBUTE TFVALUES NNZER+1 TO 8 INTO TF S 1 TO NNZER
C
      IF (NNZER.EQ.8) RETURN
C
      SUMA=0.D0
      qs=0.D0
C
      DO 30 N=1,NNZER
        SUMA=SUMA+TFVAL(N)
30    CONTINUE
C
C   FIND FRACTION OF TOTAL ACTIVE VALUE IS TFVAL(N)
      SINV=1/SUMA
      DO 40 N=1,NNZER-1
         TEMP(N)=TFVAL(N)*SINV
         qs=qs+temp(n)
40    CONTINUE
C
c***********************************************************************
c  debug - delete above refs to qs also
c
      temp(nnzer)=1-qs
C   FIND TOTAL VALUE OF TFUNCTIONS FOR INACTIVE CELLS
      SUMI=0
      DO 45 N=NNZER+1,8
        SUMI=SUMI+TFVAL(N)
45    CONTINUE

      if (abs(suma+sumi-1).gt.precno) then
          print *,'sum tfs ne 1',
     *                           sumi-1,pc,pr,pl,xhat,yhat,zhat
           stop
      endif
        val=0.D0
      do 4 m=1,nnzer
4      val=val+sumi*temp(m)
      if (abs(val-sumi).gt.precno) then
               print *,'distr-actual inact',val-sumi
                stop
        endif
c
c**********************************************************************

C   ADD FRACTION OF INACTIVE TOTAL TO ACTIVE TVALUES
      VAL=0.D0
      SUMI=1. - SUMA
      DO 50 N=1,NNZER-1
        TFVAL(N)=TFVAL(N)+SUMI*TEMP(N)
        VAL=VAL+TFVAL(N)
50    CONTINUE
      TFVAL(NNZER)=1-VAL
C
      RETURN
C
C***********************************************************************
C
C  ALL SURROUNDING CELLS INACTIVE 
C  OR (XHAT,YHAT,ZHAT) IN CENTER AREA, TF=1
5000  NNZER=1
      TFVAL(1)=1
      RETURN
C
C
      END
C*************************************************************************
      SUBROUTINE BNDYTF(PC,PR,PL,TFVAL,IBOUND,
     *           NUMTF,JS,IS,KS,J,I,K,NCOL,NROW,NLAY,
     *           NSCOL,NSROW,NSLAY,NODESS,NNZER,LBNDY,NTFACE,
     *           THCK,XFOR,XBAC,YFOR,YBAC)
C*************************************************************************
C
C     BNDYTF EVALUATES ANY NONZERO TEST FUNCTIONS AT
C     THE SPACE-TIME BOUNDARY
C
C     INPUT COORDINATES PC,PR,PL ARE FOR POINT BARELY
C     (SMALL=10-5) OUTSIDE OF SUBGRID BOUNDARY IN ONE
C     OR MORE COORDINATE DIRECTION
C     OR, UNDER UPDATED VERSION OF MOVE, EXACTLY ON SUBGRID BOUNDARY
C     WITH JS,IS,KS,J,I,K OUSIDE OF SUBGRID
C
C     NNZER   THE NUMBER OF NONZERO TEST FUNCTIONS
C     FOR N=1,NNZER
C     NUMTF(N)   BOUNDARY NODE NUMBER ASSOCIATED WITH EACH FUNCTION
C     TFVAL (N)  VALUE OF EACH NONZERO TEST FUNCTION
C
C     ARE RETURNED
C*************************************************************************
C
      PARAMETER (SMALL=1.D-5)
      PARAMETER (PRECNO=5.D-7)
      DIMENSION IBOUND(NCOL,NROW,NLAY),NUMTF(8),TFVAL(8),
     *      NSCOORD(3,8),NUMTFB(8),TFVALB(8),LBNDY(NTFACE),
     *    THCK(NSCOL,NSROW,NSLAY),XFOR(NSCOL),XBAC(NSCOL),YFOR(NSROW),
     *    YBAC(NSROW)
      DIMENSION NF(3),NEAR(3)
      COMMON /SUBGRD/ ISCOL1,ISCOL2,ISROW1,ISROW2,ISLAY1,
     *  ISLAY2,ISUBGD
C
      XHAT=PC-J
      YHAT=PR-I
      ZHAT=PL-K
      NF(1)=0
      NF(2)=0
      NF(3)=0
C
C   CHECK FOR BOUNDARY IN X DIRECTION
C   IF POINT IS ACROSS BOUNDARY, NOTE THAT: NF(1) MEANS ACROSS IN X DIR
C                                            NEAR(1) HOLDS WHICH END OF GRID
C
      IF ((JS.EQ.0).OR.(JS.EQ.NSCOL+1)) THEN
         IF (JS.EQ.0) THEN
            JS=1
            J=ISCOL1
            NEAR(1)=1
         ELSE
            JS=NSCOL
            J=ISCOL2
            NEAR(1)=0
         ENDIF
         XHAT=-XHAT
         PC=J+XHAT
         NF(1)=1
      ENDIF
C
C   NOW CHECK IF ACROSS Y BOUNDARY
C
      IF ((IS.EQ.0).OR.(IS.EQ.NSROW+1)) THEN
         IF (IS.EQ.0) THEN
            IS=1
            I=ISROW1
            NEAR(2)=1
         ELSE
            IS=NSROW
            I=ISROW2
            NEAR(2)=0
         ENDIF
         YHAT=-YHAT
         PR=I+YHAT
         NF(2)=1
      ENDIF
C
C   CHECK IF ACROSS Z BOUNDARY
C
      IF ((KS.EQ.0).OR.(KS.EQ.NSLAY+1)) THEN
         IF (KS.EQ.0) THEN
            KS=1
            K=ISLAY1
            NEAR(3)=1
         ELSE
            KS=NSLAY
            K=ISLAY2
            NEAR(3)=0
         ENDIF
         ZHAT=-ZHAT
         PL=K+ZHAT
         NF(3)=1
      ENDIF
C
C   POINT IS NOW IN TRANSPORT SUBGRID
C   IF IT'S PLACED IN AN ACTIVE CELL, GO FIND ASSOCIATED FACES
C
      IF (IBOUND(J,I,K).NE.0) GOTO 300
C
C   IF CELL IS INACTIVE, POINT IS JUST ACROSS AN INTERIOR CELL FACE
C   MOVE IT TO THE OTHER SIDE
C
      IF ((NF(1).EQ.0).AND.(ABS(XHAT).GE..5-SMALL-PRECNO)) THEN
         IF (XHAT.GT.0) THEN
            JS=JS+1
            J=J+1
         ELSE
            JS=JS-1
            J=J-1
         ENDIF
         XHAT=-XHAT
         PC=J+XHAT
      ENDIF
C
      IF (IBOUND(J,I,K).NE.0) GOTO 300
C
C   IF CELL IS INACTIVE, POINT IS JUST ACROSS AN INTERIOR CELL FACE
C   MOVE IT TO THE OTHER SIDE
C
      IF ((NF(2).EQ.0).AND.(ABS(YHAT).GE..5-SMALL-PRECNO)) THEN
         IF (YHAT.GT.0) THEN
            IS=IS+1
            I=I+1
         ELSE
            IS=IS-1
            I=I-1
         ENDIF
         YHAT=-YHAT
         PR=J+YHAT
      ENDIF
C
      IF (IBOUND(J,I,K).NE.0) GOTO 300
C
C   IF CELL IS INACTIVE, POINT IS JUST ACROSS AN INTERIOR CELL FACE
C   MOVE IT TO THE OTHER SIDE
C
      if ((nf(3).ne.0).or.(abs(zhat).lt..5-small-precno)) then
                  print *,'ERROR BNDYTF'
                        stop
        end if
      IF (ZHAT.GT.0) THEN
         KS=KS+1
         K=K+1
      ELSE
         KS=KS-1
         K=K-1
      ENDIF
      ZHAT=-ZHAT
      PL=K+ZHAT
      if (ibound(j,i,k).eq.0) then 
          print *,'error bndytf- point moved
     *         to inactive cell'
                stop
        end if

C
C   DISTRIBUTE WEIGHT TO SPACE TIME BOUNDARY NODES
C   USE EVALTF TO FIND SPACIAL BOUNDARY FACES AND DISTRIBUTE 
C   WEIGHT SPACIALLY
C
300      CALL EVALTF(PC,PR,PL,TFVALB,IBOUND,
     *           NUMTFB,JS,IS,KS,J,I,K,NCOL,NROW,NLAY,
     *           NSCOL,NSROW,NSLAY,NNZER,NSCOORD,THCK,
     *           XFOR,XBAC,YFOR,YBAC)
C
      D=NF(1)+NF(2)+NF(3)
      FRAC=1/D
      NPT2=0
C
      DO 50 NPT1=1,NNZER
      IF (NF(1).NE.0) THEN
         NPT2=NPT2+1
         NUMTF(NPT2)=NODESS+NBJ(NEAR(1),NSCOORD(2,NPT1),
     *                  NSCOORD(3,NPT1),NSROW)
         TFVAL(NPT2)=FRAC*TFVALB(NPT1)
      ENDIF
      IF (NF(2).NE.0) THEN
         NPT2=NPT2+1
         NUMTF(NPT2)=NODESS+NBI(NEAR(2),NSCOORD(1,NPT1),
     *                  NSCOORD(3,NPT1),NSCOL,NSROW,NSLAY)
         TFVAL(NPT2)=FRAC*TFVALB(NPT1)
      ENDIF
      IF (NF(3).NE.0) THEN
         NPT2=NPT2+1
         NUMTF(NPT2)=NODESS+NBK(NEAR(3),NSCOORD(1,NPT1),
     *                  NSCOORD(2,NPT1),NSCOL,NSROW,NSLAY)
         TFVAL(NPT2)=FRAC*TFVALB(NPT1)
      ENDIF

50      CONTINUE
      IF (D.EQ.3) TFVAL(3)=1-TFVAL(1)-TFVAL(2)
      NNZER=NPT2
C
      DO 55 N=1,NNZER
      IF (LBNDY(NUMTF(N)-NODESS).NE.0) THEN
c        print *,
c     *    'warning-bndytf outflow distributed to no- or inflow bndy',
c     *    ' bndy nbr=',numtf(n)-nodess,' lbndy=',LBNDY(numtf(n)-nodess)
        TFVAL(1)=TFVAL(1)+TFVAL(N)
      ENDIF
55    CONTINUE  
C
      RETURN
      END
C
